<?php

namespace Webkul\SalesforceConnector\Helper;

// use Exception;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Frontend\Pool;

class Connection extends AbstractHelper
{
    protected $_json;
    protected $writerInterface;
    protected $_curl;
    protected $_excurl;
    protected $_curlFactory;
    protected $_scopeconfig;
    protected $_productMetadata;
    public $_sfconfigData;
    protected $_setup;
    protected $messageManager;
    protected $cacheTypeList;
    protected $cacheFrontendPool;

    const REQUEST_QUERY = '/services/data/v46.0/query/?q=';
    const COMPSITE_QUERY = '/services/data/v46.0/composite/';
    const REQUEST_SOBJECT = '/services/data/v46.0/sobjects/';
    const REQUEST_REVOKE = '/services/oauth2/revoke';
    const REFRESH_TOKEN = '/services/oauth2/token?grant_type=refresh_token';
    const REQUEST_TOKEN = '/services/oauth2/token?grant_type=authorization_code';
    const APP_CONSUMER_KEY = '3MVG9ZL0ppGP5UrDmNAiEGMQGPYAYCNk.hb1xNVmqWoDi21Ug18eq6ucsSD1ZanfepgDlh0BNUF3gNjwjZaEM';
    const APP_CONSUMER_SECRET = '3093543145986445979';
    const REQUEST_WEBSERVICE_QUERY = '/services/apexrest/';

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Framework\App\Config\Storage\WriterInterface $writerInterface,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\HTTP\Adapter\CurlFactory $curlFactory,
        ExtendedCurl $exCurl,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        \Magento\Framework\Setup\SchemaSetupInterface $setup,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata
    ) {
        parent::__construct($context);
        $this->_json = $jsonHelper;
        $this->_curl = $curl;
        $this->_curlFactory = $curlFactory;
        $this->_scopeconfig = $scopeConfig;
        $this->_excurl = $exCurl;
        $this->writerInterface = $writerInterface;
        $this->_setup = $setup;
        $this->_productMetadata = $productMetadata;
        $this->messageManager = $messageManager;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
        $this->__init();
    }

    public function __init()
    {
        $restConfig = $this->_scopeconfig->getValue('sfConnect/general/restconfig');

        if ($restConfig) {
            $restConfig = $this->_json->jsonDecode($restConfig);
            if (isset($restConfig['access_token'])) {
                $this->accessToken = $restConfig['access_token'];
                $this->refreshToken = $restConfig['refresh_token'];
                $this->instanceUrl = $restConfig['instance_url'];
                $this->tokenType = $restConfig['token_type'];
                $sfConfig = $this->_scopeconfig->getValue('sfConnect/general/sfconfigdata');

                if ($sfConfig) {
                    $this->_sfconfigData = $this->_json->jsonDecode($sfConfig);
                } else {
                    if ($this->createConfigSalesforceData($restConfig)) {
                        //print sucess
                        $this->messageManager->addSuccess(__("Connection Established"));
                        $this->cacheTypeList->cleanType('config');
                        foreach ($this->cacheFrontendPool as $cacheFrontend) {
                            $cacheFrontend->getBackend()->clean();
                        }
                        header("Refresh:0");
                    } else {
                        //print failure
                        $this->messageManager->addError(__("Connection Establishment Failed Please Press Disconnect and then Reconnect"));
                        $this->cacheTypeList->cleanType('config');
                        foreach ($this->cacheFrontendPool as $cacheFrontend) {
                            $cacheFrontend->getBackend()->clean();
                        }
                        header("Refresh:0");
                    }
                }
            } else {
                $this->writerInterface->save('sfConnect/general/connection_status', '0');
            }
        } else {
            $this->writerInterface->save('sfConnect/general/connection_status', '0');
        }
    }

    public function createConfigSalesforceData($configObj)
    {
        $error = false;
        $salesforceData = [];
        $document = [];
        $sfpricebook = [];
        $sfAccrecordType = [];
        $sfContactrecordType = [];
        $sfOpprecordType = [];

        $this->writerInterface->delete('sfConnect/general/sfconfigdata');
        $this->writerInterface->delete('sfConnect/catPdt/docFolder');
        $this->writerInterface->delete('sfConnect/catPdt/pricebook');
        $this->writerInterface->delete('sfConnect/accCon/sfmainaccountdetail');
        $this->writerInterface->delete('sfConnect/accCon/guestAcc');
        $this->writerInterface->delete('sfConnect/accCon/recordAcc');
        $this->writerInterface->delete('sfConnect/accCon/recordCon');
        $this->writerInterface->delete('sfConnect/general/accountObject');
        $this->writerInterface->delete('sfConnect/ord/orderFieldMap');
        $this->writerInterface->delete('sfConnect/general/orderObject');
        $this->writerInterface->delete('sfConnect/acc/productFieldMap');
        $this->writerInterface->delete('sfConnect/general/productObject');
        $this->writerInterface->delete('sfConnect/acc/accountFieldMap');
        $this->cacheTypeList->cleanType('config');
        foreach ($this->cacheFrontendPool as $cacheFrontend) {
            $cacheFrontend->getBackend()->clean();
        }

        $urlArr = explode('/', $configObj['id']);
        $email = end($urlArr);
        $orgId = $urlArr[key($urlArr) - 1];
        $orgType = $this->_scopeconfig->getValue('sfConnect/general/orgtype');
        $response = $this->getSObject("SELECT id, Name FROM Organization WHERE id='$orgId'", 1, 1);
        $salesforceData['orgDetails'] = $response;
        if (!count($response) > 0) {
            $error = true;
        }

        $salesforceData['userName'] = $this->getSObject("SELECT id, username, email, name FROM User WHERE id='$email'");



        $contactRoles = $this->describeSObject('OpportunityContactRole');

        if (isset($contactRoles) && !empty($contactRoles) && isset($contactRoles['fields'])) {
            $salesforceData['oppContactRole'] = $contactRoles['fields'];
        }
        $query = "SELECT Id, Name, Type FROM Folder WHERE Type='Document' LIMIT 10";
        $documentFolder = $this->getSObject($query, 0);
        if (isset($documentFolder) && !empty($documentFolder)) {
            foreach ($documentFolder as $key => $doc) {
                $document[$doc['Id']] = $doc['Name'];
            }
        }
        $salesforceData['folder'] = $document;
        //$query = 'SELECT Id, Name, isStandard FROM Pricebook2 WHERE IsActive=true LIMIT 10';
        $query = 'SELECT Id, Name, isStandard FROM Pricebook2 LIMIT 10';
        $pricebook = $this->getSObject($query, 0);

        if (isset($pricebook) && !empty($pricebook)) {
            foreach ($pricebook as $key => $p_book) {
                $sfpricebook[] = [
                    'text' => $p_book['Name'],
                    'value' => $p_book['Id'],
                    'standard' => $p_book['IsStandard'],
                ];
            }
        }
        $salesforceData['pricebook'] = $sfpricebook;

        $query = "SELECT Id, Name, SobjectType FROM RecordType where SobjectType = 'Account' OR SobjectType = 'Contact' OR SobjectType = 'Opportunity' LIMIT 10";
        $allRecordType_Ids = $this->getSObject($query, 0);
        if (isset($allRecordType_Ids) && !empty($allRecordType_Ids)) {
            foreach ($allRecordType_Ids as $key => $book) {
                if ($book['SobjectType'] == 'Account' && $orgType) {
                    $sfAccrecordType[] = [
                        'text' => $book['Name'],
                        'value' => $book['Id'],
                    ];
                }
                if ($book['SobjectType'] == 'Contact') {
                    $sfContactrecordType[] = [
                        'text' => $book['Name'],
                        'value' => $book['Id'],
                    ];
                }
                if ($book['SobjectType'] == 'Opportunity') {
                    $sfOpprecordType[] = [
                        'text' => $book['Name'],
                        'value' => $book['Id'],
                    ];
                }
            }
            if (!$orgType) {
                $query = 'SELECT Id, Name FROM RecordType where IsPersonType = true';
                $allRecordType_Ids = $this->getSObject($query, 0);
                if ($allRecordType_Ids) {
                    foreach ($allRecordType_Ids as $key => $record) {
                        $sfAccrecordType[] = [
                            'text' => $record['Name'],
                            'value' => $record['Id'],
                        ];
                    }
                }
            }
            if (isset($sfOpprecordType) && !empty($sfOpprecordType)) {
                $salesforceData['OpportunityRecordType'] = $sfOpprecordType;
            }
            if (isset($sfContactrecordType) && !empty($sfContactrecordType)) {
                $salesforceData['ContactRecordType'] = $sfContactrecordType;
            }
            if (isset($sfAccrecordType) && !empty($sfAccrecordType)) {
                $salesforceData['AccountRecordType'] = $sfAccrecordType;
            }
        }

        //Account Object Detail Fetching and Saving to DB Start
        $c_url = $this->instanceUrl . '/services/data/v46.0/sobjects/Account/describe/';
        $this->_curl->get($c_url);
        $resp = $this->_curl->getBody();
        $resp = $this->_json->jsonDecode($resp);
        $finalArray = [];
        if (isset($resp['fields']) && !empty($resp['fields'])) {
            $fields = $resp['fields'];
            foreach ($fields as $field) {
                if ($field['updateable']) {
                    array_push($finalArray, ['label' => $field['label'], 'name' => $field['name'], 'type' => $field['type']]);
                }
            }
            $this->writerInterface->save('sfConnect/general/accountObject', $this->_json->jsonEncode($finalArray));
        }
        //Account Object Detail Fetching and Saving to DB End

        //Product Object Detail Fetching and Saving to DB Start
        $c_url = $this->instanceUrl . '/services/data/v46.0/sobjects/Product2/describe/';
        $this->_curl->get($c_url);
        $resp = $this->_curl->getBody();
        $resp = $this->_json->jsonDecode($resp);
        $finalArray = [];
        if (isset($resp['fields']) && !empty($resp['fields'])) {
            $fields = $resp['fields'];
            foreach ($fields as $field) {
                if ($field['updateable']) {
                    array_push($finalArray, ['label' => $field['label'], 'name' => $field['name'], 'type' => $field['type']]);
                }
            }
            $this->writerInterface->save('sfConnect/general/productObject', $this->_json->jsonEncode($finalArray));
        }
        //Product Object Detail Fetching and Saving to DB End

        //Order Object Detail Fetching and Saving to DB Start
        $c_url = $this->instanceUrl . '/services/data/v46.0/sobjects/Order/describe/';
        $this->_curl->get($c_url);
        $resp = $this->_curl->getBody();
        $resp = $this->_json->jsonDecode($resp);
        $finalArray = [];
        if (isset($resp['fields']) && !empty($resp['fields'])) {
            $fields = $resp['fields'];
            foreach ($fields as $field) {
                if ($field['updateable']) {
                    array_push($finalArray, ['label' => $field['label'], 'name' => $field['name'], 'type' => $field['type']]);
                }
            }
            $this->writerInterface->save('sfConnect/general/orderObject', $this->_json->jsonEncode($finalArray));
        }
        //Order Object Detail Fetching and Saving to DB End

        $this->writerInterface->save('sfConnect/general/sfconfigdata', $this->_json->jsonEncode($salesforceData));
        $this->_sfconfigData = $salesforceData;
        if ($error) {
            return false;
        } else {
            return true;
        }
    }

    public function getSObject($query, $limit = 1, $throwError = 0)
    {
        $headers = [
            'Authorization' => $this->tokenType . ' ' . $this->accessToken,
            'Cache-Control' => 'no-cache',
        ];
        $this->_curl->setHeaders($headers);
        $c_url = $this->instanceUrl . self::REQUEST_QUERY . urlencode($query);
        $this->_curl->get($c_url);
        $response = $this->_curl->getBody();
        $response = $this->_json->jsonDecode($response);
        if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
            if ($this->refreshToken()) {
                return $this->getSObject($query, $limit);
            } else {
                return [];
            }
        } elseif (is_array($response) && isset($response[0]['errorCode']) && !empty($response[0]['errorCode'])) {
            if ($throwError) {
                return $response;
            } else {
                return [];
            }
        } elseif (empty($response['records'])) {
            return [];
        } else {
            if ($limit == 1) {
                return $response['records'][0];
            } else {
                return $response['records'];
            }
        }
    }


    public function refreshToken()
    {
        $env = $this->_scopeconfig->getValue('sfConnect/general/enviornment');
        if ($env === 'S') {
            $refreshTokenUrl = 'https://test.salesforce.com' . self::REFRESH_TOKEN . '&client_id=' . self::APP_CONSUMER_KEY . '&client_secret=' . self::APP_CONSUMER_SECRET . '&refresh_token=' . $this->refreshToken;
        } else {
            $refreshTokenUrl = 'https://login.salesforce.com' . self::REFRESH_TOKEN . '&client_id=' . self::APP_CONSUMER_KEY . '&client_secret=' . self::APP_CONSUMER_SECRET . '&refresh_token=' . $this->refreshToken;
        }
        try {
            $this->_curl->post($refreshTokenUrl, []);
            $res = $this->_curl->getBody();
            $restConfig = $this->_json->jsonDecode($this->_scopeconfig->getValue('sfConnect/general/restconfig'));
            $res = $this->_json->jsonDecode($res);
            $restConfig['access_token'] = $res['access_token'];
            $this->accessToken = $res['access_token'];
            $this->writerInterface->save('sfConnect/general/restconfig', $this->_json->jsonEncode($restConfig));

            return true;
        } catch (\Exception $e) {
            self::eshopsyncConnLog('revoke error ' . $e->getMessage());
            $this->messageManager->addError(__("Refresh Token Failed"));
            $this->cacheTypeList->cleanType('config');
            foreach ($this->cacheFrontendPool as $cacheFrontend) {
                $cacheFrontend->getBackend()->clean();
            }
            header("Refresh:0");
            return false;
        }
    }

    public function describeSObject($sObjectName)
    {
        try {
            $c_url = $this->instanceUrl . self::REQUEST_SOBJECT . "$sObjectName/describe/";
            $this->_curl->get($c_url);
            $response = $this->_curl->getBody();
            $response = $this->_json->jsonDecode($response);
            if (is_array($response) && isset($response[0]['errorCode'])) {
                if ($response[0]['errorCode'] == 'INVALID_SESSION_ID') {
                    if ($this->refreshToken()) {
                        return $this->getSObject($query, $limit);
                    } else {
                        return false;
                    }
                }
            }

            return $response;
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());

            return [];
        }
    }

    public function revokeConnection()
    {
        try {
            $restConfig = $this->_json->jsonDecode($this->_scopeconfig->getValue('sfConnect/general/restconfig'));
            $c_url = $this->instanceUrl . self::REQUEST_REVOKE;
            $headers = [
                'Content-Type: application/x-www-form-urlencoded',
                'Cache-Control' => 'no-cache',
            ];
            $this->_curl->setHeaders($headers);
            $this->_curl->post($c_url, ['token' => $restConfig['refresh_token']]);
            $this->writerInterface->delete('sfConnect/general/restconfig');
            $this->writerInterface->delete('sfConnect/general/sfconfigdata');
            // $this->writerInterface->delete('sfConnect/catPdt/docFolder');
            // $this->writerInterface->delete('sfConnect/catPdt/pricebook');
            // $this->writerInterface->delete('sfConnect/accCon/sfmainaccountdetail');
            // $this->writerInterface->delete('sfConnect/accCon/guestAcc');
            // $this->writerInterface->delete('sfConnect/general/accountObject');
            // $this->writerInterface->delete('sfConnect/ord/orderFieldMap');
            // $this->writerInterface->delete('sfConnect/general/orderObject');
            // $this->writerInterface->delete('sfConnect/acc/productFieldMap');
            // $this->writerInterface->delete('sfConnect/general/productObject');
            // $this->writerInterface->delete('sfConnect/acc/accountFieldMap');
            $this->writerInterface->save('sfConnect/general/connection_status', '0');
            $this->cacheTypeList->cleanType('config');
            foreach ($this->cacheFrontendPool as $cacheFrontend) {
                $cacheFrontend->getBackend()->clean();
            }
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());
        }
    }

    public function insertSObject($sObject, $postFields)
    {
        try {
            $c_url = $this->instanceUrl . self::REQUEST_SOBJECT . $sObject;
            $fields = $this->_json->jsonEncode($postFields);
            $headers = [
                'Authorization' => $this->tokenType . ' ' . $this->accessToken,
                'Cache-Control' => 'no-cache',
                'Content-Type' => 'application/json',
            ];
            $this->_curl->setHeaders($headers);
            $this->_curl->post($c_url, $fields);
            $response = $this->_curl->getBody();
            $response = $this->_json->jsonDecode($response);
            if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
                if (self::refreshToken()) {
                    return $this->insertSObject($sObject, $postFields);
                }
            } elseif (isset($response['id']) && !empty($response['id'])) {
                return $response['id'];
            } elseif (is_array($response) && isset($response[0]['errorCode'])) {
                return false;
            }
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());

            return false;
        }
    }
    public function updateSObject($sObject, $patchFields, $Id)
    {
        try {
            $c_url = $this->instanceUrl . self::REQUEST_SOBJECT . $sObject . '/Id/' . $Id;

            $fields = $this->_json->jsonEncode($patchFields);
            // $headers = [
            //     'Authorization' => $this->tokenType.' '.$this->accessToken,
            //     'Cache-Control' => 'no-cache',
            //     'Content-Type' => 'application/json',
            // ];
            // $this->_curl->setHeaders($headers);
            // $this->_curl->post($c_url, $fields);
            // $response = $this->_curl->getBody();

            $response = $this->_excurl->customRequest($c_url, 'PATCH', $fields, ['tokenType' => $this->tokenType, 'accessToken' => $this->accessToken]);


            $response = $this->_json->jsonDecode($response);
            if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
                if (self::refreshToken()) {
                    return $this->insertSObject($sObject, $patchFields);
                }
            } elseif (isset($response['id']) && !empty($response['id'])) {
                return $response;
            } elseif (is_array($response) && isset($response[0]['errorCode'])) {
                return false;
            }
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());

            return false;
        }
    }


    public function deleteSobject($uri)
    {
        try {
            $c_url = $this->instanceUrl . $uri;
            $response = $this->_excurl->customRequest($c_url, 'DELETE', [], ['tokenType' => $this->tokenType, 'accessToken' => $this->accessToken]);
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());
        }
    }

    public function upsertSObject($sObject, $externalIdField, $value, $postFields)
    {
        try {
            $c_url = $this->instanceUrl . self::REQUEST_SOBJECT . $sObject . '/' . $externalIdField . '/' . $value;
            $response = $this->_excurl->customRequest($c_url, 'PATCH', $this->_json->jsonEncode($postFields), ['tokenType' => $this->tokenType, 'accessToken' => $this->accessToken]);

            $response = $this->_json->jsonDecode($response);
            if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
                if (self::refreshToken()) {
                    return $this->upsertSObject($sObject, $externalIdField, $value, $postFields);
                }
            } elseif (isset($response['id']) && !empty($response['id'])) {
                return $response['id'];
            } elseif (is_array($response) && isset($response[0]['errorCode'])) {
                self::eshopsyncConnLog($response[0]['errorCode']);

                return false;
            }
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());

            return false;
        }
    }

    public function compositeUpsertSObject($postFields)
    {
        try {
            $c_url = $this->instanceUrl . self::COMPSITE_QUERY;
            $fields = $this->_json->jsonEncode($postFields);

            $headers = [
                'Authorization' => $this->tokenType . ' ' . $this->accessToken,
                'Cache-Control' => 'no-cache',
                'Content-Type' => 'application/json',
            ];
            $this->_curl->setHeaders($headers);
            $this->_curl->post($c_url, $fields);
            $response = $this->_curl->getBody();
            $response = $this->_json->jsonDecode($response);
            if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
                if (self::refreshToken()) {
                    return $this->compositeUpsertSObject($postFields);
                }
            }

            return $response;
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());

            return false;
        }
    }

    public static function eshopsyncConnLog($message)
    {
        if ($message) {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $logger = $objectManager->create('Webkul\SalesforceConnector\Logger\EshopsyncLogger');
            $logger->critical($message);
        }
    }

    public static function checkConnectionStatus()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $scopeConfig = $objectManager->create('\Magento\Framework\App\Config\ScopeConfigInterface');

        return $scopeConfig->getValue('sfConnect/general/connection_status');
    }

    public function resetFunction()
    {
        try {
            $conn = $this->_setup->getConnection();
            $conn->truncateTable('wk_salesforce_eshopsync_product_mapping');
            $conn->truncateTable('wk_salesforce_eshopsync_order_mapping');
            $conn->truncateTable('wk_salesforce_eshopsync_lead_mapping');
            $conn->truncateTable('wk_salesforce_eshopsync_customer_mapping');
            $conn->truncateTable('wk_salesforce_eshopsync_contact_mapping');
            $conn->truncateTable('wk_salesforce_eshopsync_category_mapping');
        } catch (\Exception $e) {
            self::eshopsyncConnLog($e->getMessage());
        }
    }

    public function getMagentoEdition()
    {
        if (!empty($this->_productMetadata)) {
            return $this->_productMetadata->getEdition();
        }
    }
    //test
    public function hitApi($url, $query = '')
    {
        $headers = [
            'Authorization' => $this->tokenType . ' ' . $this->accessToken,
            'Cache-Control' => 'no-cache',
        ];

        $this->_curl->setHeaders($headers);
        $c_url = $this->instanceUrl . self::REQUEST_WEBSERVICE_QUERY . $url . urlencode($query);
        $this->_curl->get($c_url);
        $response = $this->_curl->getBody();
        $response = $this->_json->jsonDecode($response);

        // if (is_array($response) && isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
        //     if ($this->refreshToken()) {
        //         return $this->hitApi($url, $query);
        //     } else {
        //         return [];
        //     }
        // }
    }
}
