<?php
/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_SalesforceConnector
 * @author    Webkul
 * @copyright Copyright (c) 2010-2016 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Ui\Component\Listing\Column;

use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;

/**
 * Class Productlink.
 */
class Error extends Column
{
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;
    protected $_catalogProduct;
    protected $_json;
    /**
     * Constructor.
     *
     * @param ContextInterface   $context
     * @param UiComponentFactory $uiComponentFactory
     * @param array              $components
     * @param array              $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Framework\View\Asset\Repository $assetRepo,
        \Magento\Framework\App\RequestInterface $request,
        array $components = [],
        array $data = []
    ) {
        $this->_json = $jsonHelper;
        $this->assetRepo = $assetRepo;
        $this->request = $request;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * Prepare Data Source.
     *
     * @param array $dataSource
     *
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as &$item) {
                if (isset($item['created_by'])) {
                    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                    $createdByName =  $objectManager->get(\Magento\User\Model\UserFactory::class)->create()->load($item['created_by'])->getName();
                    $item['created_by'] = $createdByName;

                }
                if(isset($item['sforce_id'])){
                    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                    $restConfig = $objectManager
                            ->get('Magento\Framework\App\Config\ScopeConfigInterface')
                            ->getValue('sfConnect/general/restconfig');
                    
                    if ($restConfig) {
                        $restConfig = $this->_json->jsonDecode($restConfig);
                        if (isset($restConfig['access_token'])) {
                            $sfId=$item['sforce_id'];
                            $sfUrl=$restConfig['instance_url'].'/'.$sfId;
                            $item['sforce_id'] = "<a href='$sfUrl' target='_blank' >$sfId</a>";
                        } 
                    }
                }
                if(isset($item['opp_sforce_id'])){
                    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                    $restConfig = $objectManager
                            ->get('Magento\Framework\App\Config\ScopeConfigInterface')
                            ->getValue('sfConnect/general/restconfig');
                    
                    if ($restConfig) {
                        $restConfig = $this->_json->jsonDecode($restConfig);
                        if (isset($restConfig['access_token'])) {
                            $sfId=$item['opp_sforce_id'];
                            $sfUrl=$restConfig['instance_url'].'/'.$sfId;
                            $item['opp_sforce_id'] = "<a href='$sfUrl' target='_blank' >$sfId</a>";
                        } 
                    }
                }
                $params = ['_secure' => $this->request->isSecure()];
                $noErrorImageUrl =  $this->assetRepo->getUrlWithParams('Webkul_SalesforceConnector::Images/noerror.png', $params);
                $errorImageUrl =  $this->assetRepo->getUrlWithParams('Webkul_SalesforceConnector::Images/error.png', $params);
                $updateImageUrl =  $this->assetRepo->getUrlWithParams('Webkul_SalesforceConnector::Images/refresh.png', $params);
                if ($item['sync_status'] === null) {
                    $item['sync_status'] = "<img src='$updateImageUrl' title='need to be sync'/>";
                } elseif ($item['sync_status'] == 0) {
                    $message = $item['error'];
                   // $item['sync_status'] = "<img src=' $errorImageUrl' title='$message'/>";
                   $item['sync_status'] = '<img src="'.$errorImageUrl.'" title="'.$message.'" />';
                } elseif ($item['sync_status'] == 1) {
                    $message = $item['error'] ?? 'Sync Successfully';
                    $item['sync_status'] = "<img src='$noErrorImageUrl' title='$message'/>";
                }
            }
        }
        return $dataSource;
    }
}
